@@ -6,6 +6,8 @@
 #include <fmt/ostream.h>
 #include <boost/optional.hpp>
 
+#include "RendererSvg.h"
+
 namespace httpgd
 {
     namespace web
@@ -95,7 +97,8 @@ namespace httpgd
             return buf.str();
         }
 
-        inline bool authorized(std::shared_ptr<httpgd::HttpgdServerConfig> &m_conf, OB::Belle::Server::Http_Ctx &ctx)
+        template<typename T>
+        inline bool authorized(std::shared_ptr<httpgd::HttpgdServerConfig> &m_conf, T &ctx)
         {
             if (!m_conf->use_token)
             {
@@ -209,6 +212,51 @@ namespace httpgd
                 ctx.res.body() = json_make_state(m_watcher->api_state());
             });
 
+            m_app.on_http("/renderers", OB::Belle::Method::get, [&](OB::Belle::Server::Http_Ctx &ctx) {
+                if (!authorized(m_conf, ctx))
+                {
+                    throw OB::Belle::Status::unauthorized;
+                }
+
+                ctx.res.set("content-type", "application/json");
+                ctx.res.result(OB::Belle::Status::ok);
+
+                fmt::memory_buffer buf;
+                fmt::format_to(buf, "{{\n \"renderers\": [\n");
+                
+                const auto &renderers = RendererManager::defaults();
+
+                for (auto it = renderers.string_renderers().begin(); it != renderers.string_renderers().end(); it++) 
+                {
+                    fmt::format_to(buf, R""(  {{ "id": "{}", "mime": "{}", "ext": "{}", "name": "{}", "type": "{}", "bin": false }})"",
+                        it->second.id,
+                        it->second.mime,
+                        it->second.fileext,
+                        it->second.name,
+                        it->second.type
+                    );
+                    if (std::next(it) != renderers.string_renderers().end())
+                    {
+                        fmt::format_to(buf, ",\n");
+                    }
+                }
+                for (auto it = renderers.binary_renderers().begin(); it != renderers.binary_renderers().end(); it++) 
+                {
+                    fmt::format_to(buf, ",\n");
+                    fmt::format_to(buf, R""(  {{ "id": "{}", "mime": "{}", "ext": "{}", "name": "{}", "type": "{}", "bin": true }})"",
+                        it->second.id,
+                        it->second.mime,
+                        it->second.fileext,
+                        it->second.name,
+                        it->second.type
+                    );
+                }
+                
+                fmt::format_to(buf, "\n ]\n}}");
+
+                ctx.res.body() = fmt::to_string(buf);
+            });
+
             m_app.on_http("/plots", OB::Belle::Method::get, [&](OB::Belle::Server::Http_Ctx &ctx) {
                 if (!authorized(m_conf, ctx))
                 {
@@ -263,6 +311,19 @@ namespace httpgd
                 auto qparams = ctx.req.params();
                 auto p_width = param_double(qparams, "width");
                 auto p_height = param_double(qparams, "height");
+                double width, height, zoom;
+                if (p_width && p_height)
+                {
+                    zoom = param_double(qparams, "zoom").get_value_or(1);
+                    width = (*p_width) / zoom;
+                    height = (*p_height) / zoom;
+                } 
+                else
+                {
+                    zoom = 1;
+                    width = p_width.get_value_or(-1);
+                    height = p_height.get_value_or(-1);
+                }
                 auto p_id = param_long(qparams, "id");
 
                 boost::optional<int> index;
@@ -279,7 +340,134 @@ namespace httpgd
                 {
                     ctx.res.set("content-type", "image/svg+xml");
                     ctx.res.result(OB::Belle::Status::ok);
-                    ctx.res.body() = m_watcher->api_svg(*index, p_width.get_value_or(-1), p_height.get_value_or(-1));
+                    dc::RendererSVG renderer(boost::none);
+                    if (m_watcher->api_render(*index, width, height, &renderer, zoom)) {
+                        ctx.res.body() = renderer.get_string();
+                    } else {
+                        throw OB::Belle::Status::not_found;
+                    }
+                }
+                else
+                {
+                    throw OB::Belle::Status::not_found;
+                }
+            });
+
+            m_app.on_http("/plot", OB::Belle::Method::get, [&](OB::Belle::Server::Http_Ctx &ctx) {
+                if (!authorized(m_conf, ctx))
+                {
+                    throw OB::Belle::Status::unauthorized;
+                }
+
+                auto qparams = ctx.req.params();
+                auto p_width = param_double(qparams, "width");
+                auto p_height = param_double(qparams, "height");
+                double width, height, zoom;
+                if (p_width && p_height)
+                {
+                    zoom = param_double(qparams, "zoom").get_value_or(1);
+                    width = (*p_width) / zoom;
+                    height = (*p_height) / zoom;
+                } 
+                else
+                {
+                    zoom = 1;
+                    width = p_width.get_value_or(-1);
+                    height = p_height.get_value_or(-1);
+                }
+                auto p_id = param_long(qparams, "id");
+                auto p_renderer = param_str(qparams, "renderer").get_value_or("svg");
+                auto p_download = param_str(qparams, "download");
+
+                boost::optional<int> index;
+                if (p_id)
+                {
+                    index = m_watcher->api_index(*p_id);
+                }
+                else
+                {
+                    index = param_int(qparams, "index").get_value_or(-1);
+                }
+
+                if (index)
+                {
+                    ctx.res.set("content-type", "image/png");
+                    ctx.res.result(OB::Belle::Status::ok);
+
+                    const auto find_renderer = RendererManager::defaults().find_string(p_renderer);
+                    if (!find_renderer) {
+                        throw OB::Belle::Status::not_found;
+                    }
+                    const auto renderer = (*find_renderer).renderer();
+                    if (m_watcher->api_render(*index, width, height, renderer.get(), zoom)) {
+                        ctx.res.set("content-type", (*find_renderer).mime);
+                        if (p_download) {
+                            ctx.res.set("Content-Disposition", fmt::format("attachment; filename=\"{}\"", *p_download));
+                        }
+                        ctx.res.body() = renderer->get_string();
+                    } else {
+                        throw OB::Belle::Status::not_found;
+                    }
+                }
+                else
+                {
+                    throw OB::Belle::Status::not_found;
+                }
+            });
+            m_app.on_http("/plot", OB::Belle::Method::get, [&](OB::Belle::Server::Http_Ctx_dyn &ctx) {
+                if (!authorized(m_conf, ctx))
+                {
+                    throw OB::Belle::Status::unauthorized;
+                }
+
+                auto qparams = ctx.req.params();
+                auto p_width = param_double(qparams, "width");
+                auto p_height = param_double(qparams, "height");
+                double width, height, zoom;
+                if (p_width && p_height)
+                {
+                    zoom = param_double(qparams, "zoom").get_value_or(1);
+                    width = (*p_width) / zoom;
+                    height = (*p_height) / zoom;
+                } 
+                else
+                {
+                    zoom = 1;
+                    width = p_width.get_value_or(-1);
+                    height = p_height.get_value_or(-1);
+                }
+                auto p_id = param_long(qparams, "id");
+                auto p_renderer = param_str(qparams, "renderer").get_value_or("png");
+                auto p_download = param_str(qparams, "download");
+
+                boost::optional<int> index;
+                if (p_id)
+                {
+                    index = m_watcher->api_index(*p_id);
+                }
+                else
+                {
+                    index = param_int(qparams, "index").get_value_or(-1);
+                }
+
+                if (index)
+                {
+                    ctx.res.result(OB::Belle::Status::ok);
+
+                    const auto find_renderer = RendererManager::defaults().find_binary(p_renderer);
+                    if (!find_renderer) {
+                        throw OB::Belle::Status::not_found;
+                    }
+                    const auto renderer = (*find_renderer).renderer();
+                    if (m_watcher->api_render(*index, width, height, renderer.get(), zoom)) {
+                        ctx.res.set("content-type", (*find_renderer).mime);
+                        if (p_download) {
+                            ctx.res.set("Content-Disposition", fmt::format("attachment; filename=\"{}\"", *p_download));
+                        }
+                        ctx.res.body() = renderer->get_binary();
+                    } else {
+                        throw OB::Belle::Status::not_found;
+                    }
                 }
                 else
                 {
